NodeHooks

Struct NodeHooks 

pub struct NodeHooks {
    pub on_input: for<'a> fn(&'a mut Context, Input<AnyNode>),
    pub on_output: for<'a> fn(&'a mut Context, Output<AnyNode>),
    pub on_argument: for<'a> fn(&'a mut Context, Argument),
    pub on_result: for<'a> fn(&'a mut Context, Result),
}
Event handlers for changes being made to the node or its contained regions.

Mainly used for ensuring inputs/outputs are correctly mapped to the nodes arguments/results according to the node kinds requirements.

Fields

on_input: for<'a> fn(&'a mut Context, Input<AnyNode>)

Default: Forward this input as an argument to each contained region

on_output: for<'a> fn(&'a mut Context, Output<AnyNode>)

Default: Forward this output as an result to each contained region

on_argument: for<'a> fn(&'a mut Context, Argument)

Default: Panic, arguments may only be added implicitly by new inputs being added to the node

on_result: for<'a> fn(&'a mut Context, Result)

Default: Panic, results may only be added implicitly by new outputs being added to the node